extends Node2D

var attack_effect = preload("attack_effect.tscn")
onready var beep_timer = get_node("BeepTimer")
onready var countdown = get_node("UI/Countdown")
onready var turn_counter = get_node("UI/Turn")
onready var phase = get_node("UI/Phase")
onready var tug_1 = get_node("UI/P1")
onready var tug_2 = get_node("UI/P2")
onready var camera = get_node("Camera")
onready var tracks = get_node("Tracks")
onready var sample = get_node("SamplePlayer").get_sample_library().get_sample("beep").get_rid()
onready var bgm = get_node("StreamPlayer")
onready var sfx = get_node("SamplePlayer")
export var bpm = 120
const BPM_RATIO = 10
var spb
var count = 9
var accept_input = false
var current_bar = null
var attacking = true
var whiffed = false
var tug_of_war = 100
const BASE_DAMAGE = 10
const BAR_WIDTH = 4000
const BAR_HEIGHT = 508
const MAX_TUG = 100
var turn = 1
var player = 1
var game_over = false

func _ready():
	randomize()
	countdown.set_text("Ready?")
	calculate_spb()
	player = randi() % 2 + 1
	phase.set_text("Player " + str(player) + " Attacking")
	turn_counter.set_text("Round " + str(turn))
	if (player == 2):
		tracks.invert()
		tracks.activate(player)
	camera.set_current_animation("Camera")
	camera.seek(0.5 - 0.5*tracks.direction, true)
	tracks.reset()
	tracks.clear()
	beep_timer.set_wait_time(spb)
	beep_timer.start()
	set_process_input(true)

func _input(event):
	if (game_over and event.is_action_pressed("continue")):
		get_tree().change_scene("res://intro.tscn")
	if (event.is_action_pressed("fullscreen")):
		OS.set_window_fullscreen(not OS.is_video_mode_fullscreen())
	if (accept_input and ((event.is_action_pressed("first_1") and (player==1)) or (event.is_action_pressed("first_2") and (player==2)))):
		press(1)
	elif (accept_input and ((event.is_action_pressed("second_1") and (player==1)) or (event.is_action_pressed("second_2") and (player==2)))):
		press(2)
	elif (accept_input and ((event.is_action_pressed("third_1") and (player==1)) or (event.is_action_pressed("third_2") and (player==2)))):
		press(3)
	elif (accept_input and ((event.is_action_pressed("fourth_1") and (player==1)) or (event.is_action_pressed("fourth_2") and (player==2)))):
		press(4)

func press(key):
	if (attacking):
		if (current_bar == null):
			accept_input = false
			text_effect("Whiffed!")
			sfx.play("boo")
		else:
			current_bar.set_frame(key)
			text_effect("Huzzah!")
	else:
		if (current_bar == null):
			text_effect("Miss!")
			whiffed = true
		else:
			if (current_bar.is_active() and current_bar.get_frame() == key):
				text_effect("Hit!")
				current_bar.deactivate()
			else:
				text_effect("Wrong!")
				whiffed = true
				current_bar.deactivate()

func skip():
	text_effect("Skipped!")
	whiffed = true

func calculate_spb():
	spb = 1/(float(bpm)/60)

func text_effect(text):
	var inst = attack_effect.instance()
	inst.get_node("Label").set_text(text)
	get_node("UI/Attack"+str(player)).add_child(inst)

func play_sound(sample):
	AudioServer.voice_play(AudioServer.voice_create(), sample)

func _on_TresholdTimer_timeout():
	pass # replace with function body

func _on_BeepTimer_timeout():
	if (count > 1):
		count -= 1
		countdown.set_text("Ready?\n"+str(count))
		if (count == 8 || count == 6 || count<=4):
			AudioServer.voice_play(AudioServer.voice_create(), sample)
		if (count == 1):
			accept_input = true
			tracks.begin(spb)
	else:
		if (count == 1):
			countdown.set_text("Go!")
#		AudioServer.voice_play(AudioServer.voice_create(), sample)
	beep_timer.set_wait_time(spb)
	beep_timer.start()

func _on_AnimationPlayer_finished():
	change()

func change():
	if (attacking):
		change_player()
		phase.set_text("Player " + str(player) + " Defending")
		tracks.activate(player)
		tracks.invert()
	else:
		tracks.clear()
		turn += 1
		turn_counter.set_text("Round " + str(turn))
		bpm += BPM_RATIO
		calculate_spb()
		if (whiffed):
			damage()
			if (not game_over):
				change_player()
				tracks.invert()
		else:
			sfx.play("claps")
			tracks.reset()
		if (not game_over):
			phase.set_text("Player " + str(player) + " Attacking")
	if (not game_over):
		countdown.set_text("Ready?\n"+str(count))
		whiffed = false
		attacking = !attacking
		accept_input = false
		count = 5

func change_player():
	if (player == 1):
		player = 2
	else:
		player = 1
	tracks.activate(player)

func damage():
	var amount = BASE_DAMAGE + pow(turn,2)
	if (player == 2):
		amount *= -1
	tug_of_war += amount
	if (tug_of_war >= 200):
		tug_of_war = 200
		end_game(1)
	elif (tug_of_war <= 0):
		tug_of_war = 0
		end_game(2)
	tug_1.set_region_rect(Rect2(0,0,BAR_WIDTH-tug_of_war*20,BAR_HEIGHT))
	tug_2.set_region_rect(Rect2(0,0,tug_of_war*20,BAR_HEIGHT))

func end_game(winner):
	sfx.play("cheer")
	game_over = true
	accept_input = false
	beep_timer.stop()
	turn_counter.set_text("Player "+str(winner)+" wins!")
	phase.set_text("")
	countdown.set_text("Press Space to Restart")